#ifndef __TCPSOCKET_H
#define __TCPSOCKET_H

#include "recvform.h"
#include "sendform.h"
#include "crypto.h"

#include <qstringlist.h>
#include <qsocket.h>
#include <qfile.h>

#define TCP_PROTOCOL "vychat"
#define TCP_VERSION "1"
#define TCP_BINARY_TYPE "binary"

#define SENDING_BLOCK 8192

enum
{
    TCP_FILE_TRANSFER_READY = 1,
    TCP_FILE_TRANSFER,
    TCP_FILE_TRANSFER_DONE,
    TCP_FILE_TRANSFER_DENY,
    TCP_AVATAR_TRANSFER,
    TCP_AVATAR_TRANSFER_READY,
    TCP_BOARD_REQUEST,
    TCP_BOARD_REPLY,
    TCP_CHAT_REQUEST,
    TCP_CHAT_REQUEST_OK,
    TCP_CHAT_MSG_TYPING,
    TCP_CHAT_MSG,
    TCP_PVTMSG,
    TCP_PVTMSG_CONFIRM,
    TCP_PING,
    TCP_REQUEST_MISSED,
    TCP_MESSAGE_DELIVERY_CONFIRM,
    HTTP_GET,
    HTTP_POST
};

enum
{
    PROTOCOL_ID,
    VERSION_ID,
    MESSAGE_TYPE_ID,
    MESSAGES_ID,
    CONTENT_TYPE_ID,
    CONTENT_LENGTH_ID,
    FILE_NAME_ID,
    FILE_DESC_ID,
    FILES_LEFT_ID,
    DESC_FILES_NAME_ID,
    DESC_FILES_SIZE_ID,
    USER_NAME,
    USER_UUID_ID,
    AUTO_ANSWER_ID,
    USER_STATUS_ID,
    PVT_MESSAGE_SIG_ID,
    PVT_MASS_ID,
    DATA_SIGNATURE_ID,
    CKEY_ID,
    TYPING_MODE_ID,
    HTTP_CONTENT_LENGTH_ID,
    HTTP_CONTENT_TYPE_ID,
};

const struct
{
    const char *tname;
    char id;
} typespack[] = {
{"file-transfer-ready", TCP_FILE_TRANSFER_READY},
{"file-transfer", TCP_FILE_TRANSFER},
{"file-transfer-done", TCP_FILE_TRANSFER_DONE},
{"file-transfer-deny", TCP_FILE_TRANSFER_DENY},
{"avatar-transfer", TCP_AVATAR_TRANSFER},
{"avatar-transfer-ready", TCP_AVATAR_TRANSFER_READY},
{"board-request", TCP_BOARD_REQUEST},
{"board-reply", TCP_BOARD_REPLY},
{"chat-request", TCP_CHAT_REQUEST},
{"chat-request-ok", TCP_CHAT_REQUEST_OK},
{"chat-msg-typing", TCP_CHAT_MSG_TYPING},
{"chat-msg", TCP_CHAT_MSG},
{"pvtmsg", TCP_PVTMSG},
{"pvtmsg-confirm", TCP_PVTMSG_CONFIRM},
{"ping", TCP_PING},
{"request-missed", TCP_REQUEST_MISSED},
{"message-delivery-confirm", TCP_MESSAGE_DELIVERY_CONFIRM},
{"GET", HTTP_GET},
{"POST", HTTP_POST},
{'\0', 0}
};

const struct
{
    const char *varname;
    char id;
} varspack[] = {
    {"protocol", PROTOCOL_ID},
    {"version", VERSION_ID},
    {"message-type", MESSAGE_TYPE_ID},
    {"messages-id", MESSAGES_ID},
    {"content-type", CONTENT_TYPE_ID},
    {"content-length", CONTENT_LENGTH_ID},
    {"file-name", FILE_NAME_ID},
    {"file-desc", FILE_DESC_ID},
    {"files-left", FILES_LEFT_ID},
    {"desc-files-name", DESC_FILES_NAME_ID},
    {"desc-files-size", DESC_FILES_SIZE_ID},
    {"user-name", USER_NAME},
    {"user-uuid", USER_UUID_ID},
    {"auto-answer", AUTO_ANSWER_ID},
    {"user-status", USER_STATUS_ID},
    {"pvt-message-sig", PVT_MESSAGE_SIG_ID},
    {"pvt-mass", PVT_MASS_ID},
    {"data-signature", DATA_SIGNATURE_ID},
    {"ckey", CKEY_ID},
    {"typing-mode", TYPING_MODE_ID},
    {"Content-Length", HTTP_CONTENT_LENGTH_ID},
    {"Content-Type", HTTP_CONTENT_TYPE_ID},
    {'\0', 0}
};

class TCPSocket : public QSocket
{
    Q_OBJECT

public:
    TCPSocket( QObject *parent, CRYPTKEY *pCryptKey);
    ~TCPSocket();
    
    void createChatSession(const QHostAddress &addr, Q_UINT16 port, const QString &uid, CRYPTKEY *pubkey);
    void sleepChatSession();
    void wakeupChatSession();
    bool isChatSession() { return chatsession; }
    int version() { return vers; }
    int messageType() { return messagetype; }
    Q_ULONG contentLength() { return contentlength; }
    Q_ULONG filesLeft() { return filesleft; }
    Q_ULONG descFilesSize() { return descfilessize; }
    const QString &protocol() { return prot; }
    const QString &contentType() { return contenttype; }
    const QString &fileName() { return filename; }
    const QString &fileDesc() { return filedesc; }
    const QString &descFilesName() { return descfilesname; }
    const QString &userName() { return username; }
    const QString &userUuid() { return useruuid; }
    const QString &getSessUid() { return sessuid; }
   
    void httpGet(const QString& addr, Q_UINT16 port, const QString& uid, const QString& frq, const QString& fsd);
    void sendFile(const QStringList& filesnames, const QString& descr, const QString& recv, const QString& uuid, const QHostAddress& addr, Q_UINT16 port, bool avatar);
    void boardMsgRequest(const QHostAddress& addr, const QString& nick, const QString& uid, const QString &bids, Q_UINT16 port);
    void requestMissed(const QHostAddress& addr, const QString& nick, const QString& uid, const QString& bids, Q_UINT16 port);
    void sendMessageACK(const QHostAddress& addr, const QString& nick, const QString& uid, char status, const QString& answer, const QCString& sig, Q_UINT16 port);
    void sendMessage(const QHostAddress& addr, const QString& nick, const QString& uid, const QString &text, Q_UINT16 port);
    void sendChatMessage(const QString &text);
    void sendTypingMode(uint tmode);

signals:

    void closeConnection();
    void fileRecvFinished(const QString&, const QString&);
    void fileSendFinished(const QString&, const QString&);
    void pvtMsgReceived(const QHostAddress &, const QString &, const QString &, const QString &, const QString &, const QString &);
    void messageACKReceived(const QHostAddress&, const QString&, const QString &, const QString&, char);
    void pingReceived(const QString&, const QString&);
    void boardRequestMissedReceived(const QString&, const QString&, const QString&);
    void boardUpdated(const QByteArray &);
    void recvChatMessage(const QHostAddress&, const QString&);
    void chatSessionEstablishedByRemote(const QString&);
    void chatSessionEstablished(const QString&);
    void recvTypingMode(const QString&, uint);
    
private slots:
    void read();
    void connectDone();
    void closeConn();
    void writtenBlock(int);
    void checkConnection();
    void acceptFile(const QString& sender, const QString& fpath, bool avatar);
    void fileTransferDeny(int startfrom);

private:
    bool vis;
    bool chatsession;
    bool initialized;
    bool accepted;
    bool connectestablished;
    bool sessionestablished;
    bool httpsession;
    bool createsenderfolder;
    int vers;
    int userstatus;
    int messagetype;
    int sendtype;
    uint typingmode;
    Q_ULONG contentlength;
    Q_ULONG filesleft;
    Q_ULONG receivedbytes;
    Q_ULONG sendbytes;
    Q_ULONG descfilessize;
    Q_ULONG filenumber;
    Q_ULONG totalfiles;
    Q_ULONG currfilenum;
    Q_ULONG updlim;
    Q_ULONG currtransfer;
    QCString pvtmessagesig;
    QCString ckey;
    QString prot;
    QString mtype;
    QString contenttype;
    QString filename;
    QString fullpath;
    QString filedesc;
    QString descfilesname;
    QString useruuid;
    QString sessuid;
    QString remoteusername;
    QString username;
    QString filespath;
    QString autoanswer;
    QString pvtmass;
    QString datasignature;
    QString messages;
    QString text;
    QStringList filesnames;
    
    
    QByteArray buff;
    QStringList packet;
    QFile file;
    RecvForm* rf;
    SendForm* sf;
    
    CRYPTKEY *pkey;
    CRYPTKEY *skey;
    
    QTimer *dtimer;
    
    void sendpacket();
    void receiveMessage(const QByteArray&ba);
    void receiveBoardMessage(const QByteArray &ba);
    void receiveFile(const QByteArray &ba);
    void fileReadError();
    void writeNextBlock();
    void sendNextFile();
    void httpError();
    
//    void sessionEstablishingError();
};

#endif
