#include "userslist.h"
#include "icons.h"
#include "global.h"
#include <qpopupmenu.h>

UsersList::UsersList(QWidget *parent, const char *name): QListView(parent, name)
{
    addColumn("Total:");
    
    setShowSortIndicator (true);
    setFocusPolicy(QWidget::NoFocus);
    setSelectionMode(QListView::Single);
    setResizeMode (QListView::LastColumn);
    setItemMargin(2);
    setHScrollBarMode(QScrollView::AlwaysOff);
    setMouseTracking (true);
  
    ownmenu = new QPopupMenu(this);
    ownmenu->insertItem(tr("Change details"), ID_EDITPERSONAL);
    
    connect(ownmenu, SIGNAL(activated(int)), this, SLOT(menuAction(int)));
    
    menu = new QPopupMenu(this);
    menu->insertItem(Icons::icon_massmessage, tr("Mass Message"), ID_MASSMESSAGE);
    menu->insertSeparator();
    menu->insertItem(Icons::icon_find, tr("Find by IP"), ID_FINDBYIP);
    menu->insertSeparator();
    menu->insertItem(Icons::icon_refresh, tr("Refresh list"), ID_REFRESH);

    connect(menu, SIGNAL(activated(int)), this, SLOT(menuAction(int)));

    usermenu = new QPopupMenu(this);
    usermenu->insertItem(Icons::icon_message, tr("Message"), ID_MESSAGE);
    usermenu->insertItem(Icons::icon_private, tr("Private chat"), ID_PRVCHAT);
    usermenu->insertItem(Icons::icon_info, tr("Request info"), ID_REQINFO);
    usermenu->insertItem(Icons::icon_saynick, tr("Say \"Nick:\""), ID_SAYNICK);
    usermenu->insertSeparator();
    usermenu->insertItem(Icons::icon_beep, tr("Beep"), ID_BEEP);
    usermenu->insertSeparator();
    usermenu->insertItem(Icons::icon_find, tr("Find by IP"), ID_FINDBYIP);
    usermenu->insertSeparator();
    usermenu->insertItem(Icons::icon_send, tr("Send file"), ID_FILESEND);
    usermenu->insertSeparator();
    usermenu->insertItem(Icons::icon_massmessage, tr("Mass Message"), ID_MASSMESSAGE);
    usermenu->insertSeparator();
    usermenu->insertItem(Icons::icon_refresh, tr("Refresh list"), ID_REFRESH);

    connect(usermenu, SIGNAL(activated(int)), this, SLOT(menuAction(int)));
    connect(this, SIGNAL(contextMenuRequested(QListViewItem *, const QPoint &, int)), this, SLOT(popupContextMenu(QListViewItem*, const QPoint &, int)));
    connect(this, SIGNAL(doubleClicked(QListViewItem *, const QPoint &, int)), this, SLOT(onDoubleClicked(QListViewItem *, const QPoint &, int)));
}

UsersList::~UsersList()
{
}

bool UsersList::newNick(const QString &onick, const QString &nnick)
{
    User *user = getUser(onick);
    if (user)
    {
	user->setNick(nnick);
	umap.remove(onick);
	umap[nnick] = user;
	repaintItem(user);
	updateToolTip(user);
	return true;
    }
    return false;
}

void UsersList::updateToolTip(User *user)
{
    QString tip;
    
    tip = tr("<nobr>User: %1</nobr><br><nobr>IP: %2</nobr>").arg(user->getNick()).arg(user->getIP().toString());
    if (!user->getAutoanswer().isEmpty())
	tip.append("<br>" + user->getAutoanswer());
    
    user->setToolTip(tip);
}

User *UsersList::addUser(const QString &nk, char gender, char status, const QHostAddress &addr, const QString &fid, const QString &color)
{
    if (exists(nk))
	return NULL;

    User *u = new User(this, nk, addr, gender, fid, color);
    u->setStatus(status);
    updateToolTip(u);
    QListView::sort();
    updateTotal();
    return umap[nk] = u;
}

bool UsersList::delUser(const QString &nick)
{
    User *user = getUser(nick);
    if (user)
    {
	umap.remove(nick);
	delete user;
	updateTotal();
	return true;
    }
    return false;
}

void UsersList::clear()
{
    umap.clear();
    QListView::clear();
    updateTotal();
}

void UsersList::updateTotal()
{
    setColumnText(0, tr("Total: ") + QString::number(childCount()));
}

char UsersList::getStatus(const QString &nick)
{
    User *user = getUser(nick);
    return user != NULL ? user->getStatus() : (char)STATUS_UNKNOWN; 
}

char UsersList::getRactive(const QString &nick)
{
    User *user = getUser(nick);
    return user!=NULL ? user->getRactive() : '0'; 
}

char UsersList::getGender(const QString &nick)
{
    User *user = getUser(nick);
    return user != NULL ? user->getGender() : '0'; 
}

QString UsersList::getAvatar(const QString &nick)
{
    User *user = getUser(nick);
    return user != NULL ? user->getAvatar() : QString(""); 
}

QString UsersList::getUserColor(const QString &nick)
{
    User *user = getUser(nick);
    return user != NULL ? user->getUserColor() : QString(""); 
}

QString UsersList::getAutoanswer(const QString &nick)
{
    User *user = getUser(nick);
    return user != NULL ? user->getAutoanswer() : QString(""); 
}

bool UsersList::setStatus(const QString &nick, char s)
{
    User *user = getUser(nick);
    return user != NULL ? repaintItem(user), setStatus(user, s) : false;
}

bool UsersList::setRactive(const QString &nick, char r)
{
    User *user = getUser(nick);
    return user != NULL ? repaintItem(user), setRactive(user, r) : false;
}

bool UsersList::setGender(const QString &nick, char g)
{
    User *user = getUser(nick);
    return user != NULL ? repaintItem(user), user->setGender(g) : false;
}

bool UsersList::setAvatar(const QString &nick, const QString& avatar, bool p)
{
    User *user = getUser(nick);
    return user != NULL ? user->setAvatar(avatar, p) : false;
}

bool UsersList::setUserColor(const QString &nick, const QString& color)
{
    User *user = getUser(nick);
    return user != NULL ? user->setUserColor(color) : false;
}

bool UsersList::setAutoanswer(const QString &nick, const QString &autoanswer)
{
    User *user = getUser(nick);
    if (user != NULL)
    {
	user->setAutoanswer(autoanswer);
	updateToolTip(user);
	return true;
    }
    else
	return false;
}

QString UsersList::getNick(const QHostAddress &source)
{
    QString n("");
    for (UserMap::iterator it = umap.begin(); it!=umap.end(); ++it)
    {
	if (it.data()->getIP() == source) 
	{
	    n = it.key();
	    break;
	}
    }
    return n;
}

bool UsersList::AvatarIsAPhoto(const QString &nick)
{
    User *user = getUser(nick);
    return user!=NULL ? user->AvatarIsAPhoto() : false; 
}

QString UsersList::getToolTip(const QString &unick)
{
    User *user = getUser(unick);
    if (unick == nick) emit getLocalAutoanswer();
    return user!=NULL ? user->getToolTip() : ""; 
}

bool UsersList::setToolTip(const QString &nick, const QString &tooltip)
{
    User *user = getUser(nick);
    return user != NULL ? user->setToolTip(tooltip) : false;
}

QStringList UsersList::getNicks(const QString &source)
{
    QStringList n;
    QString tmp;
    for (UserMap::iterator it = umap.begin(); it!=umap.end(); ++it)
    {
	tmp = it.data()->getIP().toString();
	if (tmp.left(source.length()) == source)
	    n.append(it.key() + ":" + tmp);
    }
    return n;	
}

bool UsersList::resetActivityTimer(const QHostAddress &addr)
{
    User *user = getUser(getNick(addr));
    return user != NULL ? user->resetTimer() : false;
}

void UsersList::incActivityTimer()
{
    int time = 0;
    QStringList dellist;
    
    for (UserMap::iterator it = umap.begin(); it!=umap.end(); ++it)
    {
	if (it.key() != nick) {
	    time = it.data()->incTimer();
	    if ((time == FIRST_PING_LIMIT) || (time == SECOND_PING_LIMIT)) emit pingRequest(it.key());
	    if (time == WHOHERE_LIMIT) emit whohereRequest(it.key());
	    if (time >= TIMEOUT_LIMIT) 
		dellist.append(it.key());
	}
    }
    
    if (!dellist.isEmpty())
    {
	for(uint i = 0; i < dellist.count(); i++)
	    emit deleteRequest(dellist[i]);
    }
}

void UsersList::popupContextMenu(QListViewItem *it, const QPoint &p, int col)
{
    if (it) {
	if (it->text(0) == nick)
	{
	    ownmenu->popup(p);
	}
	else
	{
	    usermenu->removeItem(ID_SAYNICK);
	    usermenu->insertItem(Icons::icon_saynick, QString(tr("Say \"%1:\"")).arg(it->text(0)), ID_SAYNICK, 3);
	    usermenu->popup(p);
	}
    }
    else
	menu->popup(p);
}

void UsersList::dummy()
{
}

void UsersList::onDoubleClicked(QListViewItem *it, const QPoint &pos, int col)
{
    if (it)
	emit doubleClickedOnUser(it->text(0));
}

void UsersList::menuAction(int option)
{
    switch(option)
    {
    case ID_MASSMESSAGE:
	emit sendMassMessage();
	return;
    case ID_REFRESH:
	emit refresh();
	return;
    case ID_FINDBYIP:
	emit findByIP();
	return;
    default: break;
    }
    
    QListViewItem *it = currentItem();
    if (it)
    {
	const QString n = it->text(0);	
	switch (option)
	{
	case ID_MESSAGE: emit doubleClickedOnUser(n); break;
	case ID_REQINFO: emit requestInfo(n); break;
	case ID_BEEP:    emit beep(n); break;
	case ID_PRVCHAT: emit privateChat(n); break;
	case ID_SAYNICK: emit sayNick(n); break;
	case ID_FILESEND: emit filesend(n); break;
	case ID_EDITPERSONAL: emit editpersonal(); break;
	default: break;
	}
    }
}

QStringList UsersList::getAllUsers()
{
    QStringList allnicks;
    
    QListViewItemIterator it(this);    
    for ( ; it.current(); ++it ) 
	allnicks.append(it.current()->text(0));
    return allnicks;
}

void UsersList::updatePixmaps()
{
    for (UserMap::iterator it = umap.begin(); it!=umap.end(); ++it)
	it.data()->updatePixmap();
}

void UsersList::mouseMoveEvent ( QMouseEvent * e )
{
    emit activityDetected();
}

void UsersList::scrollToUser(const QString & name)
{
    QListViewItem *item = findItem(name, 0);
    if (item != NULL)
    {
	setCurrentItem(item);
	ensureItemVisible(item);
    }
}
